/*
 * Chapter 19
 *
 * The buttons connected to PD4,5,6 pins affect the state of PD7,PC0, PC1 pins, where LEDs are connected.
 * These states are stored in the EEPROM, the states can be restored after power cycle.
 */

#include "../Headers/main.h"

//EEPROM memory addresses for the states of each LED
const uint8_t* led1_ptr = 0x00;
const uint8_t* led2_ptr = 0x01;
const uint8_t* led3_ptr = 0x02;

int main(void)
{
    //Initializing PORTs
    IOInit();

    //Check stored state at led1_ptr address
    if (eeprom_read_byte(led1_ptr) > 0)
    {
        sbi(PORTC, 1);
    }

    //Check stored state at led2_ptr address
    if (eeprom_read_byte(led2_ptr) > 0)
    {
        sbi(PORTC, 0);
    }

    //Check stored state at led3_ptr address
    if (eeprom_read_byte(led3_ptr) > 0)
    {
        sbi(PORTD, 7);
    }

    //Infinite loop
    while (1)
    {
        //Handling the first button
		//If button is pressed
        if (tbi(PIND, 4) != 0)
        {
            //Waiting for transients to decay, software debouncing
            _delay_ms(BTN_DELAY);

            //Wait for button release
            while (tbi(PIND, 4) != 0);

            //Waiting for transients to decay, software debouncing
            _delay_ms(BTN_DELAY);

            //If LED on PC1 is "ON"
            if (tbi(PINC, 1))
            {
                //New state to be stored in "OFF"
                eeprom_write_byte(led1_ptr, 0x00);
                //Turn LED off
                cbi(PORTC, 1);
            }

            //If LED on PC1 is "OFF"
            else
            {
                //New state to be stored in "ON"
                eeprom_write_byte(led1_ptr, 0xFF);
                //Turn LED on
                sbi(PORTC, 1);
            }
        }

		//Handling the second button (same as first)
        if (tbi(PIND, 5) != 0)
        {
            _delay_ms(BTN_DELAY);

            while (tbi(PIND, 5) != 0);

            _delay_ms(BTN_DELAY);

            if (tbi(PINC, 0))
            {
                eeprom_write_byte(led2_ptr, 0x00);
                cbi(PORTC, 0);
            }

            else
            {
                eeprom_write_byte(led2_ptr, 0xFF);
                sbi(PORTC, 0);
            }
        }

		//Handling the second button (same as first)
        if (tbi(PIND, 6) != 0)
        {
            _delay_ms(BTN_DELAY);

            while (tbi(PIND, 6) != 0);

            _delay_ms(BTN_DELAY);

            if (tbi(PIND, 7))
            {
                eeprom_write_byte(led3_ptr, 0x00);
                cbi(PORTD, 7);
            }

            else
            {
                eeprom_write_byte(led3_ptr, 0xFF);
                sbi(PORTD, 7);
            }
        }
    }

    return 0;
}